from __future__ import absolute_import
from __future__ import division
from __future__ import print_function
from __future__ import unicode_literals

YELLOW = '\033[1;33m'
LRED = '\033[1;31m'
LGREEN = '\033[1;32m'
GREEN = '\033[9;32m'
WHITE = '\033[0;37m'
LWHITE = '\033[1;37m'
LCYAN = '\033[1;36m'
LBLUE = '\033[1;34m'
RESET = '\033[0m'
CRITICAL = '\033[41;37m'
